//---------------------------------------------------------------
//  File: bmpimage.cpp
//
//  Image manipulation functions for Windows BMP images.
//---------------------------------------------------------------
#include <fstream.h>
#include <limits.h>
#include <stdio.h>
#include <math.h>
#include "bmpimage.h"
#include "jpeg.h"
extern unsigned char huge *ThresData;
extern unsigned char huge *SepareThresData;
extern Threscheck;
extern long bfOffBits,bfSize,number_of_bytes,start_point;
extern int im_height,im_width;
const size_t maxread  = 30*1024; // Read 30K at a time
const size_t maxwrite = 30*1024; // Write 30K at a time
long SizeFromDepth(int w, int h, int depth);

float PixPerByte(int depth);
void define_huff_table(unsigned char *huffbits, unsigned char *huffvalues,unsigned char huge **indata);
void decode_table_gen(PTR_HUFFCODE ptr_huffcode_table, short int *maxcode,short int *mincode,
	  int *valptr,unsigned char *huffbits);
void build_huff_decode_table(int *ptr_huff_decoder);
int decode_data(short int *mincode,short int *maxcode,int *valptr,unsigned char *huffvalues,
	unsigned char huge **indata, int *bit_count);
unsigned short int nextbits(unsigned char huge **indata, int *bit_count,int bits_req);
PTR_HUFFCODE build_huffsizes(int *temp_size, unsigned char *huffbits, int size);
void build_huffcodes(PTR_HUFFCODE ptr_huffcode_table);
int predict1(unsigned char huge *indata, int w, long int pixel_num, int depth,  int pred_type);
short int categorize(short int difference);
void jpegdecomp(unsigned char huge *indata, int w, int h, int depth, unsigned char huge *outdata);

//---------------------------------------------------------------
//  B M P I m a g e : : r e a d
//  Read and interpret a Windows .BMP image (Device Independent
//  Bitmap).
int BMPImage::get_thres_image()
{
	 delete imdata;
	 imdata = new ImageData;
	 imdata->p_dib = GlobalAllocPtr(GHND, number_of_bytes);
	 if (ThresData == 0) return 0;
	 unsigned char huge *data =
					 (unsigned char huge*)imdata->p_dib;
	 for (long i=0; i<number_of_bytes;i++) data[i]=*(ThresData+i);

	 LPBITMAPINFOHEADER p_bi = (LPBITMAPINFOHEADER)imdata->p_dib;
	 imdata->bytes_per_line = ((long)p_bi->biWidth *
								 (long)p_bi->biBitCount + 31L) / 32 * 4;

	 return 1;
}
int BMPImage::read(const char* filename)
{
// If there is an existing image, detach the image data
// before reading a new image
	 if(imdata->p_dib != 0) {
	 detach();
	 GlobalFreePtr(SepareThresData);
	 }

// Open file for reading
	 ifstream ifs(filename, ios::in | ios::binary);
// Call BMPImage::read(ifstream& ifs) to read in image
	 return read(ifs);
}
//---------------------------------------------------------------
// B M P I m a g e : : r e a d ( i f s t r e a m & )
// Read image information from an open file

int BMPImage::read(ifstream& ifs)
{

	 if(!ifs)
	 {
// Error reading file. Return 0.
		  return 0;
	 }

// Read the file header
	 ifs.read((unsigned char*)&bmphdr, sizeof(BITMAPFILEHEADER));

// Check if image file format is acceptable (the type
// must be 'BM'
	 if(bmphdr.bfType != (('M' << 8) | 'B')) return 0;

// Determine size of DIB to read -- that's the file size (as
// specified by the bfSize field of the BITMAPFILEHEADER
// structure) minus the size of the BITMAPFILEHEADER
	 long bmpsize = bmphdr.bfSize - sizeof(BITMAPFILEHEADER);
// Allocate space for the bitmap
	 imdata->p_dib = GlobalAllocPtr(GHND, bmpsize);


// If memory allocation fails, return 0
	 if(imdata->p_dib == 0) return 0;

// Load the file in big chunks. We don't have to interpret
// because our internal format is also BMP.

// Allocate a large buffer to read from file
	 unsigned char *rbuf = new unsigned char[maxread];
	 if(rbuf == NULL)
	 {
		  detach();
		  return 0;
	 }
	  unsigned char huge *data = (unsigned char huge*)imdata->p_dib;
	 if (ThresData != 0)
	 {
		GlobalFreePtr(ThresData);
		ThresData=(unsigned char huge*)GlobalAllocPtr(GHND, bmpsize);
	 }
	 SepareThresData=(unsigned char huge*)GlobalAllocPtr(GHND, bmpsize);
	 unsigned int chunksize;
	 unsigned int i;

	 long z=0;
	 while(bmpsize > 0)
	 {
		  if(bmpsize > maxread)
				chunksize = maxread;
		  else
				chunksize = bmpsize;
		  ifs.read(rbuf, chunksize);

// Copy into DIB
		  for(i = 0; i < chunksize; i++){
			data[i] = rbuf[i];
			*(SepareThresData+z)=data[i];
			z++;
		  }
		  bmpsize -= chunksize;
		  data += chunksize;
	 }
	 delete rbuf;

// Compute bytes per line, rounding up to align at a 4-byte
// boundary
	 LPBITMAPINFOHEADER p_bi = (LPBITMAPINFOHEADER)imdata->p_dib;
	 imdata->bytes_per_line = ((long)p_bi->biWidth *
								 (long)p_bi->biBitCount + 31L) / 32 * 4;

// Ignore OS/2 1.x bitmap files. These files have a header
// size of 12 bytes whereas a Windows 3.1 DIB has a 40-byte
// header.
	 if(p_bi->biSize == 12) return 0;

	 return 1;
}
int BMPImage::convert(ifstream& ifs)
{
	 int w,h,depth;
	 long comp_length;
	 long filesize;
	 if(!ifs)
	 {
		  return 0;
	 }
	 ifs.read((unsigned char *)&pcthdr, sizeof(IHEAD));
	 sscanf(pcthdr.width,"%d",&w);
	 sscanf(pcthdr.height,"%d",&h);
	 sscanf(pcthdr.depth,"%d",&depth);
	 sscanf(pcthdr.complen,"%ld",&comp_length);
	 filesize=SizeFromDepth(w,h,depth);
	 start_point=sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD);
	 unsigned char huge *outdata=(unsigned char huge *)GlobalAllocPtr(GHND,filesize);
	 unsigned char huge *indata=(unsigned char huge*)GlobalAllocPtr(GHND,comp_length);
	 unsigned int chunksize;
	 unsigned int i;
	 unsigned char *rbuf = new unsigned char[maxread];
	 long z=0;
	 long temp=comp_length;
	 while(temp > 0)
	 {
		  if(temp > maxread)
				chunksize = maxread;
		  else
				chunksize = temp;
		  ifs.read(rbuf, chunksize);

		  for(i = 0; i < chunksize; i++)
		  {
			 *(indata+z) = rbuf[i];
			 z++;
		  }
		  temp -= chunksize;
	}
	delete rbuf;
	jpegdecomp(indata,w,h,depth,outdata);
	make_image(w,h,outdata);



	 return 1;
}
//---------------------------------------------------------------
//  B M P I m a g e : : w r i t e
//  Write a Windows .BMP image to a file (in Device Independent
//  Bitmap format)

int BMPImage::write(const char* filename)
{
// If there is no image, return without doing anything
	 if(imdata->p_dib == 0) return 0;

// Open file for binary write operations.
	 ofstream ofs(filename, ios::out | ios::binary);
	 if(!ofs) return 0;

// Set up BMP file header
	 bmphdr.bfType = ('M' << 8) | 'B';
	 bmphdr.bfReserved1 = 0;
	 bmphdr.bfReserved2 = 0;
	 bmphdr.bfOffBits = sizeof(BITMAPFILEHEADER) +
			  sizeof(BITMAPINFOHEADER) +
			  numcolors() * sizeof(RGBQUAD);
	 bmphdr.bfSize = (long) height() *
		  (long) imdata->bytes_per_line +
							  bmphdr.bfOffBits;

// Write the file header to the file
	 ofs.write((unsigned char*)&bmphdr, sizeof(BITMAPFILEHEADER));

// Save the file in big chunks.

// Allocate a large buffer to be used when transferring
// data to the file

	 unsigned char *wbuf = new unsigned char[maxwrite];
	 if(wbuf == NULL) return 0;
	 unsigned char huge *data =(unsigned char huge*)imdata->p_dib;

	 unsigned int chunksize;
	 long bmpsize = bmphdr.bfSize - sizeof(BITMAPFILEHEADER);
	 for (long k=0;k<bmpsize;k++) *(SepareThresData+k)=*(data+k);
	 unsigned int i;

	 while(bmpsize > 0)
	 {
		  if(bmpsize > maxwrite)
				chunksize = maxwrite;
		  else
				chunksize = bmpsize;
// Copy image from DIB to buffer
		  for(i = 0; i < chunksize; i++) wbuf[i] = data[i];
		  ofs.write(wbuf, chunksize);
		  bmpsize -= chunksize;
		  data += chunksize;
	 }
	 delete wbuf;
	 return 1;
}
int BMPImage::convert(const char* filename)
{
// If there is an existing image, detach the image data
// before reading a new image
	 if(imdata->p_dib != 0) {
	 detach();
	 GlobalFreePtr(SepareThresData);
	 }

// Open file for reading
	 ifstream ifs(filename, ios::in | ios::binary);
// Call BMPImage::read(ifstream& ifs) to read in image
	 return convert(ifs);
}

int BMPImage::get_image()
{
	 LPBITMAPINFOHEADER p_bminfo =
									(LPBITMAPINFOHEADER)(imdata->p_dib);

	 bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) +
			  numcolors() * sizeof(RGBQUAD);
	 bfSize = (long) height() *
		  (long) imdata->bytes_per_line + bfOffBits;
	 number_of_bytes = bfSize - sizeof(BITMAPFILEHEADER);
	 start_point=sizeof(BITMAPINFOHEADER) + numcolors() * sizeof(RGBQUAD);
	 SepareThresData=(unsigned char huge*)imdata->p_dib;
	 return 1;
}
long SizeFromDepth(int w, int h, int depth)
{
	 long filesize;
	 float pixperbyte;
	 pixperbyte=(float)PixPerByte(depth);
	 filesize=(long)( ceill(w/pixperbyte)*h);
	 return filesize;
}
float PixPerByte(int depth)
{
	float pixperbyte;
	switch(depth){
	case 1:
		pixperbyte = 8.0;
		break;
	case 2:
		pixperbyte = 4.0;
		break;
	case 4:
		pixperbyte = 2.0;
		break;
	case 8:
		pixperbyte = 1.0;
		break;
	case 16:
		pixperbyte = 0.5;
		break;
	case 32:
		pixperbyte = 0.25;
		break;
	}
	return pixperbyte;
}

int BMPImage::make_image(int w, int h,unsigned char huge *outdata)
{
	 BITMAPINFOHEADER bh;
	 bh.biSize        = sizeof(BITMAPINFOHEADER);
	 bh.biWidth       = w;
	 bh.biHeight      = h;
	 bh.biPlanes      = 1;
	 bh.biBitCount    = 8;
	 bh.biCompression = BI_RGB;

	 imdata->w = w;
	 imdata->h = h;

// Compute bytes per line, rounding up to align at a 4-byte
// boundary
	 imdata->bytes_per_line = ((long)bh.biWidth *
								 (long)bh.biBitCount + 31L) / 32 * 4;

	 bh.biSizeImage   = (long)bh.biHeight *
							  (long)imdata->bytes_per_line;

	 bh.biXPelsPerMeter  = 0;
	 bh.biYPelsPerMeter  = 0;

// Determine number of colors in the palette
	 short ncolors = 0;
	 switch(bh.biBitCount)
	 {
		  case 1:
				ncolors = 2;
				break;
		  case 4:
				ncolors = 16;
				break;
		  case 8:
				ncolors = 256;
				break;
		  default:
				ncolors = 0;
	 }

	 bh.biClrUsed        = ncolors;
	 bh.biClrImportant   = 0;

// Compute total size of DIB

	 unsigned long dibsize = sizeof(BITMAPINFOHEADER) +
									 ncolors * sizeof(RGBQUAD) +
									 bh.biSizeImage;
	 imdata->p_dib = GlobalAllocPtr(GHND, dibsize);
	 unsigned char huge *data=(unsigned char huge*)imdata->p_dib;
	 PALETTEENTRY ape[256];
	 LPLOGPALETTE p_pal = (LPLOGPALETTE) GlobalAllocPtr(GHND,
														  sizeof(LOGPALETTE) +
									 ncolors * sizeof(PALETTEENTRY));
	 p_pal->palVersion = 0x0300;
	 p_pal->palNumEntries = ncolors;
	 unsigned char red,green,blue;
	 int i;
	 for (i = 0, red = 0, green = 0, blue = 0; i < ncolors;i++, red += 1, green += 1, blue += 1)
	 {

		  data[40+i*4]=ape[i].peRed=p_pal->palPalEntry[i].peRed = i;//LOBYTE(red);
		  data[40+i*4+1]=ape[i].peGreen=p_pal->palPalEntry[i].peGreen =i;// LOBYTE(green);
		  data[40+i*4+2]=ape[i].peBlue=p_pal->palPalEntry[i].peBlue =i;// LOBYTE(blue);
		  data[40+i*4+3]=ape[i].peFlags=p_pal->palPalEntry[i].peFlags =0;// PC_RESERVED;
	 }
	 imdata->hpal = CreatePalette(p_pal);
	 GlobalFreePtr(p_pal);


// Allocate memory for the DIB


	 if(imdata->p_dib == NULL) return 0;
	 _fmemcpy(imdata->p_dib, &bh, (size_t)bh.biSize);


	 SepareThresData=(unsigned char huge*)GlobalAllocPtr(GHND, dibsize);
	 long z=0,v;
	 for (long k=h-1;k>=0;k--)
	 {
		for (long l=0;l<w;l++)
		{
		  v=(long)start_point+(long) k*(long) w+(long)l;
		 *(data+v)=outdata[z];
		  z++;
		}
	 }
	 for (k=start_point;k<10000;k++)
	 {
		int h=(int) data[k];
		int y=0;
	 }

	 SepareThresData=data;

	 LPBITMAPINFOHEADER p_bi = (LPBITMAPINFOHEADER)imdata->p_dib;
	 imdata->bytes_per_line = ((long)p_bi->biWidth *
								 (long)p_bi->biBitCount + 31L) / 32 * 4;

	 return 1;
}
int BMPImage::create_image(int j,int w, int h,unsigned char huge *outdata)
{
	 BITMAPINFOHEADER bh;
	 bh.biSize        = sizeof(BITMAPINFOHEADER);
	 bh.biWidth       = w;
	 bh.biHeight      = h;
	 bh.biPlanes      = 1;
	 bh.biBitCount    = 8;
	 bh.biCompression = BI_RGB;

	 imdata->w = w;
	 imdata->h = h;

// Compute bytes per line, rounding up to align at a 4-byte
// boundary
	 imdata->bytes_per_line = ((long)bh.biWidth *
								 (long)bh.biBitCount + 31L) / 32 * 4;

	 bh.biSizeImage   = (long)bh.biHeight *
							  (long)imdata->bytes_per_line;

	 bh.biXPelsPerMeter  = 0;
	 bh.biYPelsPerMeter  = 0;

// Determine number of colors in the palette
	 short ncolors = 0;
	 switch(bh.biBitCount)
	 {
		  case 1:
				ncolors = 2;
				break;
		  case 4:
				ncolors = 16;
				break;
		  case 8:
				ncolors = 256;
				break;
		  default:
				ncolors = 0;
	 }

	 bh.biClrUsed        = ncolors;
	 bh.biClrImportant   = 0;

// Compute total size of DIB

	 unsigned long dibsize = sizeof(BITMAPINFOHEADER) +
									 ncolors * sizeof(RGBQUAD) +
									 bh.biSizeImage;
	 imdata->p_dib = GlobalAllocPtr(GHND, dibsize);
	 unsigned char huge *data=(unsigned char huge*)imdata->p_dib;
	 PALETTEENTRY ape[256];
	 LPLOGPALETTE p_pal = (LPLOGPALETTE) GlobalAllocPtr(GHND,
														  sizeof(LOGPALETTE) +
									 ncolors * sizeof(PALETTEENTRY));
	 p_pal->palVersion = 0x0300;
	 p_pal->palNumEntries = ncolors;
	 unsigned char red,green,blue;
	 int i;
	 if (j==0)
	 {
		for (i = 0, red = 0, green = 0, blue = 0; i < ncolors;i++, red += 1, green += 1, blue += 1)
		{

		  data[40+i*4]=ape[i].peRed=p_pal->palPalEntry[i].peRed = i;//LOBYTE(red);
		  data[40+i*4+1]=ape[i].peGreen=p_pal->palPalEntry[i].peGreen =i;// LOBYTE(green);
		  data[40+i*4+2]=ape[i].peBlue=p_pal->palPalEntry[i].peBlue =i;// LOBYTE(blue);
		  data[40+i*4+3]=ape[i].peFlags=p_pal->palPalEntry[i].peFlags =0;// PC_RESERVED;
		}
	 }
	 else
	 {
		 unsigned char *PointR=(unsigned char *)GlobalAllocPtr(GHND,4);
		 unsigned char *PointGB=(unsigned char *)GlobalAllocPtr(GHND,8);
		 PointR[0]=0;PointR[1]=64;PointR[2]=128;PointR[3]=255;
		 PointGB[0]=0;PointGB[1]=32;PointGB[2]=64;PointGB[3]=96;
		 PointGB[4]=128;PointGB[5]=160;PointGB[6]=192;PointGB[7]=255;
		 int k1=0,l=0;
		 for (i = 0; i < ncolors;i++)
		 {
			data[40+i*4]=ape[i].peRed=p_pal->palPalEntry[i].peRed = PointR[k1];
			k1++;
			if (k1>3) k1=0;
		 }
		 k1=0;l=0;
		 for (i = 0; i < ncolors;i++)
		 {
			data[40+i*4+1]=ape[i].peGreen=p_pal->palPalEntry[i].peGreen =PointGB[k1];
			l++;
			if (l>3)
			{
				k1++;l=0;
			}
			if (k1>7) k1=0;
		 }
		 for (i=0;i<ncolors;i++)
		 {
			data[40+i*4+2]=ape[i].peBlue=p_pal->palPalEntry[i].peBlue =PointGB[k1];
			data[40+i*4+3]=ape[i].peFlags=p_pal->palPalEntry[i].peFlags =0;
			l++;
			if(l>31)
			{
			  k1++;l=0;
			}
			if (k1>7) k1=0;
		 }
	 }
	 imdata->hpal = CreatePalette(p_pal);
	 GlobalFreePtr(p_pal);


// Allocate memory for the DIB


	 if(imdata->p_dib == NULL) return 0;
	 _fmemcpy(imdata->p_dib, &bh, (size_t)bh.biSize);


	 SepareThresData=(unsigned char huge*)GlobalAllocPtr(GHND, dibsize);
	 long z=0,v;
	 for (long k=0;k<h;k++)
	 {
		for (long l=0;l<w;l++)
		{
		  v=(long)start_point+(long) k*(long) w+(long)l;
		 *(data+v)=outdata[z];
		  z++;
		}
	 }


	 SepareThresData=data;

	 LPBITMAPINFOHEADER p_bi = (LPBITMAPINFOHEADER)imdata->p_dib;
	 imdata->bytes_per_line = ((long)p_bi->biWidth *
								 (long)p_bi->biBitCount + 31L) / 32 * 4;

	 return 1;
}


void jpegdecomp(unsigned char huge *indata, int w, int h, int depth, unsigned char huge *outdata)
{
	unsigned int diff_code;
	int diff_cat;
	long int pixel;
	int full_diff_code;
	unsigned char pred_type;
	long num_pixels;
	int last_size;
	unsigned char *huffbits=(unsigned char *)GlobalAllocPtr(GHND,MEMSIZE2);
	unsigned char *huffvalues=(unsigned char *)GlobalAllocPtr(GHND,MEMSIZE2);
	PTR_HUFFCODE ptr_huffcode_table;

	short int *maxcode=(short int *)GlobalAllocPtr(GHND,sizeof(short int)*MEMSIZE1);
	short int *mincode=(short int *)GlobalAllocPtr(GHND,sizeof(short int)*MEMSIZE1);
	int *valptr=(int *)GlobalAllocPtr(GHND,sizeof(int)*MEMSIZE1);
	int bit_count = 0;
	int data_pred;
	int *ptr_huff_decoder=(int *) GlobalAllocPtr(GHND,5120*sizeof(int));

	num_pixels = (long) w * (long) h;

	define_huff_table(huffbits, huffvalues, &indata);

	pred_type = *indata++;

	ptr_huffcode_table = build_huffsizes(&last_size, huffbits, MEMSIZE1);
	build_huffcodes(ptr_huffcode_table);


	decode_table_gen(ptr_huffcode_table, maxcode, mincode, valptr, huffbits);
	free(ptr_huffcode_table);

	build_huff_decode_table(ptr_huff_decoder);

	for(pixel = 0; pixel < num_pixels; pixel++) {


		diff_cat = decode_data(mincode, maxcode, valptr,
								  huffvalues, &indata, &bit_count);

		diff_code = nextbits(&indata, &bit_count, diff_cat);

		full_diff_code = ptr_huff_decoder[diff_cat*512+diff_code];

		data_pred = predict1(outdata, w, pixel, depth, pred_type);
		*outdata = full_diff_code + data_pred;
		outdata++;
	}



	 return;
}
void define_huff_table(huffbits, huffvalues, indata)
	unsigned char *huffbits;		/*defines number of huffman codes
					  of each code length*/
	unsigned char *huffvalues;		/*defines order of huffman code
					  lengths in relation to code sizes*/
	unsigned char huge **indata;		/*compressed input data*/

{
	int inx;				/*increment variable*/
	unsigned short int number;		/*number of huffbits and huffvalues*/

	number = *(*indata)++;

	for (inx = 0; inx < MEMSIZE2;  inx++)
		huffbits[inx] = *(*indata)++;
	for (inx = 0; inx < (number - MEMSIZE2); inx ++)
		huffvalues[inx] = *(*indata)++;
}


/*********************************************/
/*routine to generate tables needed to decode*/
/*********************************************/

void decode_table_gen(ptr_huffcode_table, maxcode, mincode, valptr, huffbits)
	PTR_HUFFCODE ptr_huffcode_table;	/*table of huffman codes and sizes*/
	short int *maxcode;			/*points to maximum code value for
					  a given code length*/
	short int *mincode;			/*points to minimum code value for
					  a given code length*/
	int *valptr;			/*points to first code in the huffman
					  code table for a given code length*/
	unsigned char *huffbits;		/*defines number of huffman codes
					  of each code length*/

{
	int inx, inx2 = 0;			/*increment variables*/

   for(inx = 1; inx <= MEMSIZE2; inx++) {
      if(huffbits[inx - 1] == 0) {
	 maxcode[inx] = -1;
	 continue;
      }
      valptr[inx] = inx2;
      mincode[inx] = (ptr_huffcode_table + inx2) -> code;
      inx2 = inx2 + huffbits[inx - 1] - 1;
		maxcode[inx] = (ptr_huffcode_table + inx2) -> code;
		inx2++;
	}
}


/***************************************************/
/*Routine to build code table for difference values*/
/***************************************************/

void build_huff_decode_table(int *ptr_huff_decoder)
  //	int ptr_huff_decoder[10][512];  /*holds the code for all possible
  //						  difference values that occur
  //					  when encoding*/

{
	int cat, count, diff;
	int bit, difftemp;

	for(count = SMALLESTDIFF; count < LARGESTDIFF; count++) {
		diff = count;
		cat = categorize(diff);
		if(diff < 0) {
			diff--;
	 difftemp = 0;
			for(bit = 0; bit < cat; bit++){
		 if(((diff >> bit) & LSBITMASK) != 0)
			 difftemp |= (LSBITMASK << bit);
	 }
			diff = difftemp;
		}

		ptr_huff_decoder[cat*512+diff] = count;
	}
		return;
}


/************************************/
/*routine to decode the encoded data*/
/************************************/

int decode_data(mincode, maxcode, valptr, huffvalues, indata, bit_count)
	short int *mincode;		/*points to minimum code value for
				  a given code length*/
	short int *maxcode;		/*points to maximum code value for
				  a given code length*/
	int *valptr;		/*points to first code in the huffman
				  code table for a given code length*/
	unsigned char *huffvalues;	/*defines order of huffman code
				  lengths in relation to code sizes*/
	unsigned char huge **indata;	/*compressed input data*/
	int *bit_count;		/*marks the bit to receive from the input byte*/

{
   int inx, inx2;		/*increment variables*/
   int code;			/*becomes a huffman code word
				  (one bit at a time)*/
   int diff_cat;		/*category of the huffman code word*/

	code = nextbits(indata, bit_count, 1);

   for(inx = 1; code > maxcode[inx]; inx++)
		code = (code << 1) + (nextbits(indata, bit_count, 1));
   inx2 = valptr[inx];
   inx2 = inx2 + code - mincode[inx];
   diff_cat = huffvalues[inx2];
   return diff_cat;
}


/******************************************/
/*routine to get nextbit(s) of data stream*/
/******************************************/

unsigned short int nextbits(indata, bit_count, bits_req)
	unsigned char huge **indata;	/*compressed input data*/
	int *bit_count;		/*marks the bit to receive from the input byte*/
	int bits_req;		/*number of bits requested*/

{
	static unsigned char code;	/*next byte of data*/
   unsigned short int bits;	/*bits of current data byte requested*/
   int bits_needed;		/*additional bits required to finish request*/

				/*used to "mask out" n number of
				  bits from data stream*/
   static unsigned char bit_mask[9] = {0x00,0x01,0x03,0x07,0x0f,
				       0x1f,0x3f,0x7f,0xff};

   if(*bit_count == 0) {
      code = *(*indata)++;
      *bit_count = BYTE;
	}
   if(bits_req <= *bit_count) {
      bits = (code >>(*bit_count - bits_req)) & (bit_mask[bits_req]);
		*bit_count -= bits_req;
      code &= bit_mask[*bit_count];
   }
   else {
      bits_needed = bits_req - *bit_count;
      bits = code << bits_needed;
      *bit_count = 0;
      bits |= nextbits(indata, bit_count, bits_needed);
	}
   return bits;
}


/**************************************************************************/
/*This routine defines the huffman code	sizes for each difference category*/
/**************************************************************************/

PTR_HUFFCODE build_huffsizes(temp_size, huffbits, size)
	int *temp_size;			/*storage of current code size*/
	unsigned char *huffbits;		/*defines number of huffman codes
					  of each code length*/
	int size;				/*size of huffbits array*/

{
	PTR_HUFFCODE ptr_huffcode_table;	/*table of huffman codes and sizes*/
	int code_size;			/*code sizes*/
   int number_of_codes = 1;		/*the number codes for a given
					  code size*/

   if((ptr_huffcode_table = (PTR_HUFFCODE)calloc(size,
      sizeof(HUFFCODE))) == NULL)
		printf("build_huffsizes", "calloc", "ptr_huffcode_table");

	*temp_size = 0;

   for(code_size = 1; code_size < size; code_size++) {
		while(number_of_codes <= huffbits[code_size - 1]) {
	 (ptr_huffcode_table + *temp_size)->size = code_size;
         (*temp_size)++;
	 number_of_codes++;
      }
   number_of_codes = 1;
   }
   (ptr_huffcode_table + *temp_size)->size = 0;
   return ptr_huffcode_table;
}


/****************************************************************************/
/*This routine defines the huffman codes needed for each difference category*/
/****************************************************************************/

void build_huffcodes(ptr_huffcode_table)
	PTR_HUFFCODE ptr_huffcode_table;	/*table of huffman codes and sizes*/

{
	int pointer = 0;			/*pointer to code word information*/
	int temp_code = 0;			/*used to construct code word*/
	int temp_size;			/*used to construct code size*/

	temp_size = ptr_huffcode_table->size;

	do {
		do {
			(ptr_huffcode_table + pointer)->code = temp_code;
			temp_code++;
			pointer++;
		}
		while((ptr_huffcode_table + pointer)->size == temp_size);

		if((ptr_huffcode_table + pointer)->size == 0)
         return;

		do {
			temp_code <<= 1;
			temp_size++;
		}
		while((ptr_huffcode_table + pointer)->size != temp_size);
	}
	while((ptr_huffcode_table + pointer)->size == temp_size);
}


/**************************************************************/
/*This routine is used to predict1 the pixel values in an image*/
/**************************************************************/

int predict1(indata, w, pixel_num, depth, pred_type)
	unsigned char huge *indata;
	int w, depth;
	long int pixel_num;
	int pred_type;

{
	int data_pred;
	int multiples;

	if(pixel_num == 0) {
		multiples = depth - 1;
		for(data_pred = 1; multiples > 0; --multiples)
			data_pred *= BASE;
		return data_pred;
	}

	if(pixel_num > 0 && pixel_num < w) {
		data_pred = *(indata - 1);
		return data_pred;
	}

	if(pixel_num > (w - 1))  {
		if((pixel_num % w) == 0)
	 data_pred = *(indata - w);

		else
	 switch(pred_type) {
		 case 1:
			 data_pred = *(indata - 1);
			 break;
		 case 2:
			 data_pred = *(indata - w);
			 break;
		 case 3:
					data_pred = *(indata -(w + 1));
			 break;
		 case 4:
			 data_pred = *(indata - 1) +
									*(indata - w) -
									*(indata -(w + 1));
			 break;
		 case 5:
			 data_pred = *(indata - 1) +
									((*(indata - w) >> 1) -
									(*(indata -(w + 1)) >> 1));
			 break;
		 case 6:
			 data_pred = *(indata - w) +
									((*(indata - 1) >> 1) -
									(*(indata -(w + 1)) >> 1));
			 break;
		 case 7:
			 data_pred = (*(indata -1) + *(indata - w)) / 2;
			 break;
				default:
		  /*	 printf("predict1",
								"Invalid predict1ion type.  Must be a value 1-7.",
								(char *) NULL);*/
			 break;
			}
	}
	return data_pred;
}


/********************************************************************/
/*This function determines the category for a given difference value*/
/********************************************************************/

short int categorize(difference)
	short int difference;	/*difference to categorize*/

{
	int bit;			/*bit pointer to difference value*/

	if(difference == 0)
		return (0);   		/*difference category zero*/

	if (difference < 0)
		difference *= -1;

	for (bit = 0; bit < SHORTCODELENGTH; bit++) {
		if ((difference & CATMASK) != 0)
		 return (SHORTCODELENGTH - bit);
		difference <<= 1;
	}
}


