  //----------------------------------------------------------------
 //  File: image.cpp
//
//  Image manipulation functions
//---------------------------------------------------------------
#include <fstream.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "image.h"
#include <alloc.h>
//extern BOOL Threscheck;
// extern unsigned char huge *ThresData;
// extern unsigned char huge *SepareThresData;
extern long bfOffBits,bfSize,number_of_bytes,start_point;
extern int im_width,im_height;
const size_t maxwrite = 30*1024; // Write 30K at a time
//---------------------------------------------------------------
//  I m a g e D a t a : : ~ I m a g e D a t a
//  Destructor for an Image.


ImageData::~ImageData()
{

// If a DIB exists, delete it.
	 if(p_dib != 0) GlobalFreePtr(p_dib);

// If a palette exists, free it also.
	 if(hpal != 0) DeletePalette(hpal);

// If a DDB exists, destroy it.
	 if(hbm_ddb != 0) DeleteBitmap(hbm_ddb);

}
//---------------------------------------------------------------
//  I m a g e : : I m a g e
//  Copy constructor

Image::Image(const Image& img)
{
	 img.imdata->count++;
	 if(--imdata->count <= 0) delete imdata;
	 imdata = img.imdata;
}
//---------------------------------------------------------------
//  I m a g e : : i m a g e _ d a t a
//  Copy the ImageData pointer from another image

void Image::image_data(const Image* img)
{
	 img->imdata->count++;
	 if(--imdata->count <= 0) delete imdata;
	 imdata = img->imdata;
}
//---------------------------------------------------------------
//  I m a g e : : o p e r a t o r =
//  Assignment operator

Image& Image::operator=(const Image& img)
{

	 img.imdata->count++;
	 if(--imdata->count <= 0) delete imdata;
	 imdata = img.imdata;
	 return *this;
}
//---------------------------------------------------------------
// I m a g e : : n u m c o l o r s
// Returns the number of colors used. Returns 0 if image uses
// 24-bit pixels.

unsigned int Image::numcolors()
{

	 if(imdata->p_dib == 0) return 0;
	 LPBITMAPINFOHEADER p_bminfo =
					 (LPBITMAPINFOHEADER)(imdata->p_dib);

// If the biClrUsed field is nonzero, use that as the number of
// colors
	 if(p_bminfo->biClrUsed != 0)
		  return (unsigned int)p_bminfo->biClrUsed;

// Otherwise, the number of colors depends on the bits per pixel
	 switch(p_bminfo->biBitCount)
	 {
		  case 1: return 2;
		  case 4: return 16;
		  case 8: return 256;
		  default: return 0; // Must be 24-bit/pixel image
	 }
}
//---------------------------------------------------------------
// I m a g e : : m a k e _ p a l e t t e
// Create a color palette using information in the DIB

void Image::make_palette()
{
// Set up a pointer to the DIB
	 LPBITMAPINFOHEADER p_bminfo =
									(LPBITMAPINFOHEADER)(imdata->p_dib);
	 if(p_bminfo == 0) return;

// Free any existing palette
	 if(imdata->hpal != 0) DeletePalette(imdata->hpal);

// Set up the palette, if needed
	 if(numcolors() > 0)
	 {
		  LPLOGPALETTE p_pal = (LPLOGPALETTE) GlobalAllocPtr(GHND,
														  sizeof(LOGPALETTE) +
									 numcolors() * sizeof(PALETTEENTRY));

		  if(p_pal)
		  {
				p_pal->palVersion = 0x0300;
				p_pal->palNumEntries = numcolors();

// Set up palette entries from DIB
				LPBITMAPINFO p_bi = (LPBITMAPINFO)p_bminfo;
				int i;
				for(i = 0; i < numcolors(); i++)
				{
					 p_pal->palPalEntry[i].peRed =
								p_bi->bmiColors[i].rgbRed;
					 p_pal->palPalEntry[i].peGreen =
								p_bi->bmiColors[i].rgbGreen;
					 p_pal->palPalEntry[i].peBlue =
								p_bi->bmiColors[i].rgbBlue;
					 p_pal->palPalEntry[i].peFlags = 0;
				}
				imdata->hpal = CreatePalette(p_pal);
				GlobalFreePtr(p_pal);
		  }
	 }
}
//---------------------------------------------------------------
// I m a g e : : D I B t o D D B
// Create a device dependent bitmap from the DIB

void Image::DIBtoDDB(HDC hdc)
{
// Set up a pointer to the DIB
	 char Buffer[80];
	 LPBITMAPINFOHEADER p_bminfo =
									(LPBITMAPINFOHEADER)(imdata->p_dib);
	 if(p_bminfo == 0) return;

	 bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) +
			  numcolors() * sizeof(RGBQUAD);
		bfSize = (long) height() *
		  (long) imdata->bytes_per_line + bfOffBits;
		number_of_bytes = bfSize - sizeof(BITMAPFILEHEADER);
		start_point=sizeof(BITMAPINFOHEADER) + numcolors() * sizeof(RGBQUAD);
		if(imdata->hbm_ddb != 0) DeleteBitmap(imdata->hbm_ddb);

		im_width=imdata->bytes_per_line;
		im_height=height();

// Build the device-dependent bitmap.

// Set up pointer to the image data (skip over BITMAPINFOHEADER
// and palette).

			  LPSTR p_image = (LPSTR)p_bminfo +
						  sizeof(BITMAPINFOHEADER) +
						  numcolors() * sizeof(RGBQUAD);

// Realize palette, if there is one. Note that this does not do
// anything on the standard 16-color VGA driver because that
// driver does not allow changing the palette, but the new palette
// should work on Super VGA displays.

	 HPALETTE hpalold = NULL;
	 if(imdata->hpal)
	 {
		  hpalold = SelectPalette(hdc, imdata->hpal, FALSE);
		  RealizePalette(hdc);
	 }

// Convert the DIB into a DDB (device dependent bitmap) and
// block transfer (blt) it to the device context.
	 HBITMAP hbm_old;
	 imdata->hbm_ddb = CreateDIBitmap(hdc,
											p_bminfo,
											CBM_INIT,
											p_image,
											(LPBITMAPINFO)p_bminfo,
											DIB_RGB_COLORS);

// Don't need the palette once the bitmap is converted to DDB
// format.
	 if(hpalold) SelectPalette(hdc, hpalold, FALSE);

}

//---------------------------------------------------------------
// I m a g e : : D D B t o D I B
// Create a device independent bitmap from the DDB
// soecified by imdata->hbm_ddb

void Image::DDBtoDIB()
{
	 char Buffer[80];
	 
// Do nothing if the DDB does not exist or if the DIB exists
	 if(imdata->hbm_ddb == 0) return;
	 if(imdata->p_dib != NULL) return;

	 BITMAP bm;
	 GetObject(imdata->hbm_ddb, sizeof(bm), (LPSTR)&bm);

// Set up a BITMAPINFOHEADER for the DIB
	 BITMAPINFOHEADER bh;
	 bh.biSize        = sizeof(BITMAPINFOHEADER);
	 bh.biWidth       = bm.bmWidth;
	 bh.biHeight      = bm.bmHeight;
	 bh.biPlanes      = 1;
	 bh.biBitCount    = (WORD)(bm.bmPlanes * bm.bmBitsPixel);
	 bh.biCompression = BI_RGB;

	 imdata->w = bm.bmWidth;
	 imdata->h = bm.bmHeight;

// Compute bytes per line, rounding up to align at a 4-byte
// boundary
	 imdata->bytes_per_line = ((long)bh.biWidth *
								 (long)bh.biBitCount + 31L) / 32 * 4;

	 bh.biSizeImage   = (long)bh.biHeight *
							  (long)imdata->bytes_per_line;

	 bh.biXPelsPerMeter  = 0;
	 bh.biYPelsPerMeter  = 0;

// Determine number of colors in the palette
	 short ncolors = 0;
	 switch(bh.biBitCount)
	 {
		  case 1:
				ncolors = 2;
				break;
		  case 4:
				ncolors = 16;
				break;
		  case 8:
				ncolors = 256;
				break;
		  default:
				ncolors = 0;
	 }

	 bh.biClrUsed        = ncolors;
	 bh.biClrImportant   = 0;

// Compute total size of DIB
	 unsigned long dibsize = sizeof(BITMAPINFOHEADER) +
									 ncolors * sizeof(RGBQUAD) +
									 bh.biSizeImage;

// Allocate memory for the DIB
	 imdata->p_dib = GlobalAllocPtr(GHND, dibsize);

	 if(imdata->p_dib == NULL) return;

// Set up palette
	 HDC hdc = GetDC(NULL);

// Copy BITMAPINFO structure bh into beginning of DIB.
	 _fmemcpy(imdata->p_dib, &bh, (size_t)bh.biSize);

	 LPSTR p_image = (LPSTR)imdata->p_dib +
						  (WORD)bh.biSize +
						  ncolors * sizeof(RGBQUAD);

// Call GetDIBits to get the image and fill the palette indices
// into a BITMAPINFO structure
	 GetDIBits(hdc, imdata->hbm_ddb, 0, (WORD)bh.biHeight,
				  p_image,(LPBITMAPINFO)imdata->p_dib,
				  DIB_RGB_COLORS);
	 wsprintf(Buffer,"Colors..:%d",DIB_RGB_COLORS);
	 TextOut(hdc,0,0,Buffer,strlen(Buffer));

// All done. Clean up and return.
	 ReleaseDC(NULL, hdc);
}

//---------------------------------------------------------------
// I m a g e : : s h o w (H D C, ... )
// Display a DIB on a Windows device specified by a
// device context

void Image::show(HDC hdc, short xfrom, short yfrom,
								  short xto,   short yto,
								  short wdth,  short hght,
								  DWORD ropcode)
{
// Set up a pointer to the DIB
	 LPBITMAPINFOHEADER p_bminfo =
									(LPBITMAPINFOHEADER)(imdata->p_dib);
	 if(p_bminfo != NULL)
	 {
// Set up the palette, if needed
		  if(imdata->hpal == 0 && numcolors() > 0) make_palette();
// Convert to DDB, if necessary
		  if(imdata->hbm_ddb == 0) DIBtoDDB(hdc);
	 }

// "Blit" the DDB to hdc
	 if(imdata->hbm_ddb != 0)
	 {
		  HDC memdc = CreateCompatibleDC(hdc);
		  if(memdc != 0)
		  {
				HBITMAP hbm_old = SelectBitmap(memdc,
														 imdata->hbm_ddb);
// If width or height is zero, use corresponding dimension
// from the image.
				if(wdth == 0) wdth = width();
				if(hght == 0) hght = height();

				BitBlt(hdc, xto, yto, wdth, hght,
						 memdc, xfrom, yfrom, ropcode);
				SelectBitmap(memdc, hbm_old);
				DeleteDC(memdc);
		  }
	 }
}
//---------------------------------------------------------------
// I m a g e : : s h o w ( T D C &, ... )
// Display a DIB on a Windows device specified by a
// device context

void Image::show(TDC& dc, short xfrom, short yfrom,
								  short xto,   short yto,
								  short wdth,  short hght,
								  DWORD ropcode)
{
// Set up a pointer to the DIB
	 char Buffer[80];
	 LPBITMAPINFOHEADER p_bminfo =
									(LPBITMAPINFOHEADER)(imdata->p_dib);
	 if(p_bminfo != NULL)
	 {
// Set up the palette, if needed
		  if(imdata->hpal == 0 && numcolors() > 0) make_palette();

// Convert to DDB, if necessary
		  if(imdata->hbm_ddb == 0) DIBtoDDB(dc);
	 }

// "Blit" the DDB to hdc
	 if(imdata->hbm_ddb != 0)
	 {
		  TMemoryDC memdc(dc);
		  if(memdc != 0)
		  {
				memdc.SelectObject(imdata->hbm_ddb);

// If width or height is zero, use corresponding dimension
// from the image.

				if(wdth == 0) wdth = width();
				if(hght == 0) hght = height();

				dc.BitBlt(xto, yto, wdth, hght,
							 memdc, xfrom, yfrom, ropcode);
				memdc.RestoreBitmap();
		  }
	 }


}


//---------------------------------------------------------------
// Image::I m a g e
// Construct an image by copying a portion of the bitmap from
// another image

Image::Image(HDC hdc, Image *img, short x, short y,
				 unsigned short w, unsigned short h)
{
	 imdata = new ImageData;
	 if(img == NULL) return;

	 unsigned short iw = img->width();
	 unsigned short ih = img->height();

	 if(x < 0) x = 0;
	 if(y < 0) y = 0;

// If width or height is 0, adjust them
	 if(w == 0) w = iw;
	 if(h == 0) h = ih;

// Make sure width and height are not too large
	 if((w+x) > iw) w = iw - x;
	 if((h+y) > ih) h = ih - y;

// Save width and height
	 imdata->w = w;
	 imdata->h = h;

// Create a new bitmap for the new image
	 imdata->hbm_ddb = CreateCompatibleBitmap(hdc, w, h);
	 if(imdata->hbm_ddb != 0)
	 {
		  HDC memdcn = CreateCompatibleDC(hdc);
		  HDC memdco = CreateCompatibleDC(hdc);
		  if(memdcn != 0 && memdco != 0)
		  {
				HBITMAP ohbm = SelectBitmap(memdco, img->get_ddb());
		 HBITMAP nhbm = SelectBitmap(memdcn, imdata->hbm_ddb);
		 BitBlt(memdcn, 0, 0, w, h, memdco, x, y, SRCCOPY);
		 SelectBitmap(memdco, ohbm);
				SelectBitmap(memdcn, nhbm);
		 DeleteDC(memdco);
		 DeleteDC(memdcn);
		  }
	 }
}
//---------------------------------------------------------------
// Image:: w r i t e _ d i b
// Write out the DIB starting at the current location in
// a stream (assumed to be opened with ios::out | ios::binary)

 int Image::write_dib(ofstream& ofs)
{
	 DDBtoDIB();
// If there is no image, return without doing anything
	 if(imdata->p_dib == NULL) return 0;

// Check if file is ok
	 if(!ofs) return 0;

// Set up BMP file header
	 BITMAPFILEHEADER bfhdr;

	 bfhdr.bfType = ('M' << 8) | 'B';
	 bfhdr.bfReserved1 = 0;
	 bfhdr.bfReserved2 = 0;
	 bfhdr.bfOffBits = sizeof(BITMAPFILEHEADER) +
							  sizeof(BITMAPINFOHEADER) +
							  numcolors() * sizeof(RGBQUAD);
	 bfhdr.bfSize = (long) height() *
						 (long) imdata->bytes_per_line +
											  bfhdr.bfOffBits;

// Write the file header to the file
	 ofs.write((unsigned char*)&bfhdr, sizeof(BITMAPFILEHEADER));

// Save the file in big chunks.

// Allocate a large buffer to be used when transferring
// data to the file

	 unsigned char *wbuf = new unsigned char[maxwrite];
	 if(wbuf == NULL) return 0;

	 unsigned char huge *data =
					 (unsigned char huge*)imdata->p_dib;
	 unsigned int chunksize;
	 long bmpsize = bfhdr.bfSize - sizeof(BITMAPFILEHEADER);

	 unsigned int i;
	 while(bmpsize > 0)
	 {
		  if(bmpsize > maxwrite)
				chunksize = maxwrite;
		  else
				chunksize = bmpsize;
// Copy image from DIB to buffer
		  for(i = 0; i < chunksize; i++) {
		  wbuf[i] = data[i];
		  }
		  ofs.write(wbuf, chunksize);
		  bmpsize -= chunksize;
		  data += chunksize;
	 }
	 delete wbuf;
	 return 1;
}


