//---------------------------------------------------------------
//  File: image.h
//
//  Defines the Image class.
//
//---------------------------------------------------------------
#if !defined(__IMAGE_H)
#define __IMAGE_H

#include <owl\dc.h>
#include <fstream.h>
#include <windowsx.h>

class Image;
class TGAImage;
class BMPImage;
class PCXImage;
class GIFImage;
class TIFImage;

// This class represents the data for an image
class ImageData
{
friend Image;
friend TGAImage;
friend BMPImage;
friend PCXImage;
friend GIFImage;
friend TIFImage;

protected:
	 ImageData() : p_dib(0), count(1), hpal(0), hbm_ddb(0),
						bytes_per_line(0), w(0), h(0) {}

	 ~ImageData();

protected:

// This points to a BITMAPINFOHEADER followed by the
// image data.
	 LPVOID         p_dib;   // Device independent bitmap
	 HPALETTE       hpal;    // Color palette
	 HBITMAP        hbm_ddb; // Device dependent bitmap

	 unsigned short w, h;    // Width and height
	 unsigned short bytes_per_line;

	 unsigned short count;
};

// Abstract base class for all images
class Image
{
public:
// Constructors
	 Image()
	 {
		  imdata = new ImageData;
	 }
	 Image(HBITMAP hbm, unsigned short w, unsigned short h)
	 {
		  imdata = new ImageData;
		  imdata->hbm_ddb = hbm;
		  imdata->w = w;
		  imdata->h = h;
	 }

	 Image(HDC hdc, Image *img, short x, short y,
			 unsigned short w, unsigned short h);

// Copy Constructor
	 Image(const Image& img);

	 virtual ~Image()
	 {
		  if(--imdata->count <= 0) delete imdata;
	 }

// Operators
	 Image& operator=(const Image& img);

// Copy the imdata pointer from another image
	 void image_data(const Image* img);

// Functions to load and save images
	 virtual int read(const char* filename){ return 0;}
	 virtual int convert(const char* filename){ return 0;}
	 virtual int get_thres_image(){ return 0; }
	 virtual int write(const char* filename){ return 0;}
	 virtual int read(ifstream& ifs){ return 0;}
	 virtual int convert(ifstream& ifs){ return 0;}
	 virtual int make_image(int j, int w, int h,unsigned char huge *outdata){return 0;}
	 virtual int create_image(int j, int w, int h,unsigned char huge *outdata){return 0;}
	 int write_dib(ofstream& ofs);

// Returns pointer to the Windows Device Independent
// Bitmap (DIB).
	 LPVOID get_dib() { return imdata->p_dib;}

// Function to return the handle to the device dependent
// bitmap
	 HBITMAP get_ddb() { return imdata->hbm_ddb;}

	 unsigned short width()
	 {
			 if(imdata->p_dib != 0)
				return((LPBITMAPINFOHEADER)imdata->p_dib)->biWidth;
			 return imdata->w;
	 }
	 unsigned short height()
	 {
		  if(imdata->p_dib != 0)
			 return ((LPBITMAPINFOHEADER)imdata->p_dib)->biHeight;
		  return imdata->h;
	 }

	 int image_loaded()
	 {
		  if(imdata->p_dib == 0) return 0;
		  else return 1;
	 }

	 void detach()
	 {
		  if(--imdata->count == 0) delete imdata;
		  imdata = new ImageData;
	 }

// Functions to make palette and convert to DDB
	 void make_palette();
	 void DIBtoDDB(HDC hdc);

	 void show(HDC hdc, short xfrom = 0, short yfrom = 0,
							  short xto = 0,   short yto = 0,
							  short width = 0, short height = 0,
							  DWORD ropcode = SRCCOPY);
	 void show(TDC& hdc, short xfrom = 0, short yfrom = 0,
							  short xto = 0,   short yto = 0,
							  short width = 0, short height = 0,
							  DWORD ropcode = SRCCOPY);

	 void DDBtoDIB();
	 unsigned int numcolors();
	 HPALETTE palette() { return imdata->hpal;}

protected:
	 ImageData* imdata;

};

#endif
