
//---------------------------------------------------------------
// File: imvwwin.h
//
// Classes for an OWL application that lets you open an image
// file and view the image in a window.
//---------------------------------------------------------------
#if !defined(__IMVWWIN_H)
#define __IMVWWIN_H

#include <owl\opensave.h>
#include <owl\applicat.h>
#include <owl\dialog.h>
#include <owl\dc.h>
#include <owl\gdiobjec.h>
#include <owl\mdi.h>
#include <owl\mdichild.h>
#include "image.h"
#include "bmpimage.h"
#include <strstrea.h>
#include <string.h>

#include <owl\button.h>
#include <owl\static.h>
#include <owl\scrollba.h>
#include <owl\slider.h>

#include <owl\color.h>

#include <cstring.h>

// Stop collecting precompiled header information
#pragma hdrstop

#define IDM_ABOUT         200
#define IDM_OPENFILE      101
#define IDM_SAVE          102
#define IDM_SAVEAS        103
#define IDM_THRESHOLD     104
#define IDM_INVERT        105
#define IDM_WINDOW16x16   1
#define IDM_WINDOW16x8    2
#define IDM_WINDOW8x8     106
#define IDM_WINDOW8x4     107
#define IDM_WINDOW4x4     108
#define IDM_WINDOW4x2     109
#define IDM_PROCESS       3
#define IDM_THIN_MORF     4
#define IDM_THIN_RIDGE    5
#define IDM_MEDIAN_3x3    6
#define IDM_MEDIAN_5x5    7
#define IDM_ROTATION      110
#define IDM_SCALE         111
#define IDM_PIXEL         112
#define IDM_BLOCK         113
#define IDM_MAPP          114
#define IDM_HISTEQU       8
#define IDM_COMPRESS      47
#define IDM_UNCOMPRESS    30
#define IDM_MEAN3x3       10
#define IDM_MEAN5x5       15
#define IDM_FFT           11
#define IDM_IFFT          12
#define IDM_DAUB_FWWT     24
#define IDM_DAUB_IWWT     25
#define IDM_HAAR_FWWT     26

#define IDM_HAAR_IWWT     27
#define IDM_SHOR_FWWT     28

#define IDM_SHOR_IWWT     29
#define IDM_COMP4         31
#define IDM_COMP16        32
#define IDM_COMP26_2      33
#define IDM_CUBIC         80
#define IDM_FEATURE       81

#define IDM_LOWPASS       19

void Find_Direction(int def);


class ImageViewWindow : public TMDIChild
{
public:
	 ImageViewWindow(TMDIClient &parent, const char far *title,
						  char far *fname);

	 ~ImageViewWindow()
	 {
		  if(filename != NULL) delete[] filename;
		  if(image != NULL) delete image;
	 }

	 void Paint(TDC& dc, BOOL, TRect&);
	 void Save();
	 void SaveAsFile();
	 void get_current_image();
	 void make_image(int j, int w, int h);
	 Image *current_image() { return image;}
private:
	 char    *filename;
	 Image   *image;
};

class ImageViewFrame: public TMDIClient
{
public:
	 ImageViewFrame() : TMDIClient() {}

// Declare functions for handling messages from Windows
	 BOOL EvQueryNewPalette();
	 void EvPaletteChanged(HWND hWndPalChg);
	 void OpenFile();
	 void SaveFile();
	 void SaveAsFile();
	 void Invert();
	 void About();
	 void Thres();
	 void Window16x16();
	 void Window16x8();
	 void Window8x8();
	 void Window8x4();
	 void Window4x4();
	 void Window4x2();
	 void ProcessingTime();
	 void Feature_ext();
	 void Thinning_morf();
	 void Thinning_Ridge();
	 void Median_3x3();
	 void Median_5x5();
	 void Rotation();
	 void Scale();
	 void Pixel_dir();
	 void Block_dir();
	 void Mapp();
	 void HistEqu();
	 void Compress();
	 void Uncompress();
	 void Mean3x3();
	 void Mean5x5();
	 void Fft();
	 void LowPass_Fft();
	 void Ifft();
	 void Daub_Fwt();
	 void Comp4_1();
	 void Comp16_1();
	 void Comp26_1();
	 void Daub_Iwt();
	 void Haar_Fwt();
	 void Haar_Iwt();
	 void Shor_Fwt();
	 void Shor_Iwt();
	 void Cubic_Tr();
	 void get_current_image();
private:
  int id;
  friend class TLevelOptionDialog;
DECLARE_RESPONSE_TABLE(ImageViewFrame);
};



#endif

