/*
 *
 *    P - W A V E L E T S  --  wavelet.h
 *
 *  Copyright (C) 1994  Eric L. Veum
 *
 *  You may use this code as you wish if credit is given and this message
 *  is retained.
 *
 *  Please e-mail any useful additions to the author so they may be
 *  included in later releases.
 *
 */

/***************************************************************************************
                                        NOTICE

Use extreme caution when altering this structure.  Since pointers are back to back in 
memory, and since some arrays use shifted indexing, mal-addressing any of these arrays
will destoy data in its neighbors, hence giving core dumps, and other misleading 
bug errors. Just be careful.

****************************************************************************************/
#include "linalg.h"

typedef struct {
int	N;		/* number of points */

int	*kmax;		/* boundary point for each scale left and right endpoints in phase-space */
int	*kmin;
int    termscale;      /* terminal convergence scale of boundary iterations */
int    suppscale;      /* lowest scale whose support is completely contained in the window */

double  h;              /* step size of the sampling grid in time */
double  *frequency;     /* the frequency of the different scales */

} dimension;


typedef struct {

int	p;		/* prime base */
int	n;		/* number of vanishing moments */
int	Tau;		/* max index on masking coefficients */
int	jmax;		/* Maximium and Min Scales */
int	jmin;

dimension x, y;

double  a;              /* Endpoints [a,b] of time sequence */
double  b; 
double  **psi;          /* the basis functions under inversion at the lowest scale */  
int	*width;		/* index support width of wavelet Psi[j,m,k] wrt to sampling grid  */
int    *spacing;       /* index spacing of coefficients at each level wrt sampling grid */

double  q;              /* sequence normalization used */
double	error;		/* sequence error in q-norm */

double  epsilon;	/* chopper epsilon */
double  M;              /* Maximum absolute value of coefficients */
int	compression;	/* Actual Compression Ratio */
int	count;		/* number of coefficients saved for reconstruction */
int    total;          /* total number of coefficients */

double	**H;		/* masking coefficients */
Matrix  **V;        /* coefficients Spaces V */
Matrix  *tmp;		/* work space */

char	*inputfile;	/* input data file  */

} WAVELET;




